// Local headers
#include "PsychrometricCalculator.h"

// STL headers
#include <cmath>

// Constants
const double PsychrometricCalculator::R_      = 8.3144621;     // constant of ideal gas in J/K/mol
const double PsychrometricCalculator::mAir_   = 28.965338e-3;  // molar mass of air   in kg/mol
const double PsychrometricCalculator::mWater_ = 18.0153e-3;    // molar mass of water in kg/mol
const double PsychrometricCalculator::w_      = mWater_/mAir_; // ratio of molar masses


double PsychrometricCalculator::IsenthalpicFunction(double t, double h, double p0)
{
  double rsat1 = ( VapourPressureFunction(t)*w_ ) / (p0 - VapourPressureFunction(t));
  double tref = 273.15; // ref for enthalpy = 0 degree C
  double rsat2 = (h-1.006*(t-tref))/(2501+1.805*(t-tref));
  return rsat1-rsat2;
}


// Pa
double PsychrometricCalculator::VapourPressureFunction(double t)
{
  // Buck equation [Buck (1981), J. Appl. Meteorol., 20: 1527-1532]
  // t-=273.15; // moving to degree C
  // return 611.21*std::exp((18.678-t/234.5)*(t/(257.14+t)));
  if (t<273.15) // saturation pressure over ice, validity : from -100 to 0 degree C
  {
    return std::exp( -5.6745359e+03/t + 
                      6.3925247e+00 + 
                     -9.6778430e-03*t + 
                      6.2215701e-07*t*t +
                      2.0747825e-09*t*t*t +
                     -9.4840240e-13*t*t*t*t +
                      4.1635019e+00*std::log(t) );
  }
  else // saturation pressure over liquid water, validity : from 0 to +200 degree C
  {
    return std::exp( -5.8002206e+03/t + 
                      1.3914993e+00 + 
                     -4.8640239e-02*t + 
                      4.1764768e-05*t*t +
                     -1.4452093e-08*t*t*t +
                      6.5459673e+00*std::log(t) );
  }
}


// GetVapourPressure in Pa
double PsychrometricCalculator::GetVapourPressure() const
{
  return VapourPressureFunction(t_);
}


// GetRelativeHumidity in [0,1]
double PsychrometricCalculator::GetRelativeHumidity() const
{
  // degree of saturation
  double mu = (ah_/w_)*(p0_/GetVapourPressure()-1);

  // relative humidity
  return mu/(1-(1-mu)*(GetVapourPressure()/p0_));
}


// GetSpecificEnthalpy in [kJ/kg dry air]
double PsychrometricCalculator::GetSpecificEnthalpy() const
{
  double tref = 273.15; // ref for enthalpy = 0 degree C
  return ah_*2501 +            // latent heat at 0 degree C for water
         ah_*1.805*(t_-tref) + // heat capacity for water 
         1.006*(t_-tref);      // heat capacity for air
}


// GetSpecificVolume in [m^3/kg dry air]
double PsychrometricCalculator::GetSpecificVolume() const
{
  return (R_*t_)/(mAir_*p0_)*(1+ah_/w_);

  //  return (R_*t_)*(1+r_*w_)/(mAir_*p0_);
  //  return (R_*t_)*(1+1.6078*r_)/(28.964e-3*p0_);
}


// GetDensity in [kg dry air/m^3]
double PsychrometricCalculator::GetDensity() const
{
  return 1/GetSpecificVolume();
}


// GetWetTemperature in [degree C]
double PsychrometricCalculator::GetWetTemperature() const
{
  // Getting the specific isenthalpy
  double h = GetSpecificEnthalpy();

  // Getting vapour pressure for an isenthalpic transformation
  double a = -100. + 273.15;
  double b = +200. + 273.15;
  double m = 0;
  double epsilon = 0.01;
  while ( (b-a) > epsilon)
  {
    m=(a+b)/2.;
    if ((IsenthalpicFunction(m,h,p0_)*IsenthalpicFunction(a,h,p0_))<=0) b=m; else a=m;
  }
  return (a+b)/2. - 273.15;
}

// GetDewTemperature in [degree C]
double PsychrometricCalculator::GetDewTemperature() const
{
  // Getting vapour pressure for the absolute humidity r_
  double psat = (ah_*p0_)/(ah_+w_);

  // Returning temperature corresponding to psat
  double a = -100. + 273.15;
  double b = +200. + 273.15;
  double m = 0;
  double epsilon = 0.01;
  while ( (b-a) > epsilon)
  {
    m=(a+b)/2.;
    if (VapourPressureFunction(m)>psat) b=m; else a=m;
  }
  return (a+b)/2. - 273.15;
}


bool PsychrometricCalculator::SetTemperatureRelativeHumidityPressure(double t, double rh, double p0)
{
  p0_=p0*100000.;
  if (t<-20 || t>90)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the dry-bulb temperature is outside of the area of validity: T=" 
              << t << " degree C not in [-20,+90]" << std::endl;
    return false;
  }
  if (rh<0 || rh>1)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the relative humidity is outside of the area of validity: HR=" 
              << rh*100 << "% not in [0%,100%]" << std::endl;
    return false;
  }
  t_ = t + 273.15;
  ah_ = rh*(1-(VapourPressureFunction(t_)/p0_));
  ah_ /= (p0_ - VapourPressureFunction(t_)) / (w_*VapourPressureFunction(t_));
  ah_ /= (1-rh*VapourPressureFunction(t_)/p0_);
  return true;
}

bool PsychrometricCalculator::SetTemperatureAbsoluteHumidityPressure(double t, double ah, double p0)
{
  p0_=p0*100000.;
  if (t<-100 || t>200)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the dry-bulb temperature is outside of the area of validity: T=" 
              << t << " degree C not in [-100,+200]" << std::endl;
    return false;
  }
  if (ah<0)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the absolute humidity is outside of the area of validity: r=" 
              << ah << " kg water / kg dry air not positive" << std::endl;
    return false;
  }

  // computing absolute humidity corresponding to psat
 double  rsat = (VapourPressureFunction(t+273.15)*w_)/(p0_-VapourPressureFunction(t+273.15));
  if (ah>rsat)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the absolute humidity is outside of the area of validity: r=" 
              << ah << " kg water / kg dry air greater than the absolute humidity at Psat = " 
              << rsat << " kg water / kg dry air" << std::endl;
    return false;
  }

  t_ = t + 273.15;
  ah_ = ah;

  // ok
  return true;
}

bool PsychrometricCalculator::SetTemperaturesPressure(double dry, double humid, double p0)
{
  p0_=p0*100000.;
  if (dry<-100 || dry>+200)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the dry-bulb temperature is outside of the area of validity: T=" 
              << dry << " degree C not in [-100,+200]" << std::endl;
    return false;
  }
  if (humid<-100 || humid>+200)
  {
    std::cerr << "PsychrometricCalculator: "
              << "the humid-bulb temperature is outside of the area of validity: T=" 
              << humid << " degree C not in [-100,+200]" << std::endl;
    return false;
  }

  // dry-bulb temperature
  t_ = dry + 273.15;

  // psat corresponding to the humid temperature
  double psat = VapourPressureFunction(humid + 273.15);

  // calculating absolute humidity at humid T and psat
  double ah = (psat*w_)/(p0_-psat);

  // calculating specific enthalpy at humid T and psat
  double tref = 273.15; // ref for enthalpy = 0 degree C
  double h = ah*2501 +                      // latent heat at 0 degree C for water
             ah*1.805*(humid+273.15-tref) + // heat capacity for water 
             1.006*(humid+273.15-tref);     // heat capacity for air
  
  // calculating absolute humidity at dry T and h
  ah_ = (h-1.006*(t_-tref))/(2501+1.805*(t_-tref));

  // ok
  return true;
}

void PsychrometricCalculator::Print() const
{
}

double PsychrometricCalculator::GetPressure() const
{ return p0_/100000.; }

double PsychrometricCalculator::GetDryTemperature() const
{ return t_-273.15; }

double PsychrometricCalculator::GetTemperature() const
{ return t_-273.15; }

double PsychrometricCalculator::GetAbsoluteHumidity() const
{ return ah_; }
