#ifndef PSYCHROMETRIC_CALCULATOR_H
#define PSYCHROMETRIC_CALCULATOR_H


// STL headers
#include <iostream>

class PsychrometricCalculator
{
 private :

  // data member
  double t_;   /// dry bulb temperature [K]
  double ah_;  /// absolute humidity    [kg water / kg dry air]
  double p0_;  /// air pressure         [Pa]
  
  // physics constants
  static const double R_;      /// constant of ideal gas in J/K/mol
  static const double mAir_;   /// molar mass of air in kg/mol
  static const double mWater_; /// molar mass of water in kg/mol
  static const double w_;      /// ratio of molar masses: water / air


 public :

  /// Constructor with no argument
  PsychrometricCalculator()
  { Clear(); }

  /// Destructor
  ~PsychrometricCalculator()
  { }  

  /// Reset the content of the class
  void Clear()
  { t_=0; ah_=0; p0_=0; }
  
  /// Printing the values of the data members
  void Print() const;

  /// Vapour pressure [Pa] in function as temperature [K]
  static double VapourPressureFunction(double t);
  
  /// specific enthalpy [J/kg] in function as temperature [K}
  static double IsenthalpicFunction(double t, double h, double p0);

  /// Setting the properties of the humid air by providing the dry temperature [degree C], the relative humidity [no unit] and the pressure [bar]
  bool SetTemperatureRelativeHumidityPressure(double t, double rh, double p0);
  
  /// Setting the properties of the humid air by providing the dry temperature [degree C], the absolute humidity [kg water / kg dry air] and the pressure [bar]
  bool SetTemperatureAbsoluteHumidityPressure(double t, double ah, double p0);
  
  /// Setting the properties of the humid air by providing the dry temperature [degree C], the wet temperature [degree C] and the pressure [bar]
  bool SetTemperaturesPressure(double dry, double humid, double p0);

  /// Getting the (dry) temperature [degree C]
  double GetDryTemperature()   const;
  
  /// Same than GetDryTemperature()
  double GetTemperature()      const;

  /// Getting the pressure [bar]
  double GetPressure()         const;
  
  /// Getting the absolute humidity [kg water / kg dry air]  
  double GetAbsoluteHumidity() const;
  
  /// Getting the water vapour pressure [Pa]   
  double GetVapourPressure()   const;
  
  /// Getting the air relative humidity [no unit]
  double GetRelativeHumidity() const;
  
  /// Getting the air specific enthalpy [J / kg dry air] 
  double GetSpecificEnthalpy() const;
  
  /// Getting the air specific volume [m^3 / kg dry air]
  double GetSpecificVolume()   const;
  
  /// Getting the air density [kg dry air / m^3]
  double GetDensity()          const;
  
  /// Getting the air wet temperature [degree C]
  double GetWetTemperature()   const;
  
  /// Getting the air dew temperature [degree C]
  double GetDewTemperature()   const;

};


#endif
